import numpy as np
import scipy.signal as signal
import matplotlib.pyplot as plt

# The following function calculates the group delay
# from the phase response
# w=angular frequency axis ("omega")
# p=phase data (radians)
# returns the new ang. freq. axis (wr and the delays in seconds)

def analog_group_delay(w,p):
    d=-np.diff(p)/np.diff(w)
    wr=w[:-1]
    return wr,d

# In this script we compare a bessel filter with a Butterworth with similar
# frequency points
# Note that the scipy-signal module does not has design functions for
# Bessel filters starting from the magnitude response
# Therefore, an approximate attempt to match the feauters of the
# Butterworth filter is made

# Properties of the Butterworth filter

fp=1e3
fs=5e3
Ap=1
As=40



ws=2*np.pi*fs
wp=2*np.pi*fp

# Order determination of the Butterworth filter

N,wn=signal.buttord(wp, ws, Ap, As,analog=True)

print "Order",N
print "Normalization frequency",wn/(2*np.pi)

# Design of the Butterworth filter

b1,a1=signal.butter(N, wn,analog=True)

# Attempt to match the butterworth filter by increasing
# the order of the Bessel filter by two units
# Note: the Bessel filter has the same wn (-3 dB ang freq)
# norm=mag is added to make the "bessel" design function
# consider wn as a corner frequency of the magnitude response
# otherwise, wn is considered the target group delay

b2,a2=signal.bessel(N+2, wn,analog=True,norm="mag") 


# Calculation of the frequency response

w1,mdb1,p1=signal.bode((b1,a1), n=10000)
w2,mdb2,p2=signal.bode((b2,a2), n=10000)

plt.semilogx(w1/2/np.pi,mdb1,label="butter")
plt.semilogx(w2/2/np.pi,mdb2,label="bessel")
plt.title("Frequency response (magnitude)")
plt.xlabel("frequency (Hz)")
plt.legend()
plt.show()

# Calculation and plot of the group delay

w1,d1=analog_group_delay(w1,p1/180*np.pi)
w2,d2=analog_group_delay(w2,p2/180*np.pi)
plt.semilogx(w1/2/np.pi,d1,label="butter")
plt.semilogx(w2/2/np.pi,d2,label="bessel")
plt.title("Group delay")
plt.xlabel("frequency (Hz)")
plt.xlabel("delay (s)")
plt.legend()
plt.show()

tbase=np.linspace(0,2e-3,4000)


t1,y1=signal.step2((b1,a1),T=tbase)
t2,y2=signal.step2((b2,a2),T=tbase)


plt.plot(t1,y1,label="butter")
plt.plot(t2,y2,label="bessel")
plt.legend()
plt.xlabel("time (s)")


plt.title("Step response")
plt.show()
