import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import scipy.signal as signal
import matplotlib as mpl

mpl.rc('xtick', labelsize=16)
mpl.rc('ytick', labelsize=16)
mpl.rc("axes",labelsize=18)


# Synthesis of second order low pass order DT filter by
# transformation of a corresponding CT filterm
# Uses scipy function "cont2discrete" that allows more
# flexibility with respect to the function "bilinear", being
# possible to specify different methods. 

# Available methods:
# "bilinear" (identical to the function bilinear)
# "zoh" : (default) zero order hold (optimum time domain approximation)
# "euler" : deriving from the forward Euler integrator
# "backward_diff" : deriving from the backward Euler integrator

# These are the parameters of the 2nd order system:
# w0 = characteristic frequency Q0: quality factor

Q0=3
f0=0.1

# general parameters of the discrete time system
fs=2            # Sampling frequency 
mtd="zoh"       # Method
warp=1          # if "1" applies pre-warping active only for bilinear method 

T=1.0/fs
w0=2*np.pi*f0

# Continuous time representation: b,a = numerator(s) , denominator(s)

a=np.array([1,w0/Q0,w0*w0])
b=np.array([w0*w0,])

# Freq. response and impulse response of the CT system
w,p=signal.freqs(b,a,worN=np.logspace(-2,2,500)*w0)
t,y=signal.step((b,a),T=np.linspace(0,20/w0,1000))

# Synthesis of the Discrete Time system from the CT model
# through bilinear transformation
# Set warp=1 to apply pre-warping before DT synthesis
# Note: the original CT system response is unchanged


#w,p=signal.freqs(b,a,worN=np.logspace(-2,2,500)*w0) # Freq response
#t,y=signal.step((b,a),T=np.linspace(0,T*50,1000)) # Step response

# synthesis using the cont2discrete function
# note that the output is in anusual format (maybe due to a bug),
# so that the operation shown after the function call is necessary to get bz



if (warp==1) and (mtd=="bilinear"):   
    w0_w=2*fs*np.tan(w0/fs/2)
    a=np.array([1,w0_w/Q0,w0_w*w0_w])
    b=np.array([w0_w*w0_w,])
    #bz,az=signal.bilinear(b_w,a_w,fs)


sist=signal.cont2discrete((b,a),T,method=mtd)
bz=sist[0][0]
az=sist[1]


wz,pz=signal.freqz(bz,az) # Freq response
tz,yz=signal.dstep([bz,az,T],n=int(20/w0/T)) # Step response


plt.loglog(w/2/np.pi,abs(p),label="CT")
plt.loglog(wz/2/np.pi*fs,abs(pz),label=mtd)
plt.xlim(0,0.5/T)
plt.legend(bbox_to_anchor=(0.3, 1))
plt.show()

plt.plot(t,y,label="CT")
plt.plot(tz,yz[0],label=mtd)

plt.legend(loc=4)

plt.show()
